var urls = {
  nascom: {
    dev: 'http://99859448.kevin.vanbreda-microsites.dev02.nascomdev.be/m/dashboard?wrapped=1',
    acc: 'http://cigna-corporate.acc01.nascomacc.be/splashpage-mobile?wrapped=1'
  },

  cigna: {
    test: 'https://test.cignahealthbenefits.com/splashpage-mobile?wrapped=1',
    stag: 'https://staging.cignahealthbenefits.com/splashpage-mobile?wrapped=1',
    prod: 'https://cignahealthbenefits.com/splashpage-mobile?wrapped=1'
  }
};

var app = window.app = {
  cordova: false,
  deviceready: false,
  debug: true,
  totalPhotos: 0,
  claimPhotos: [],
  environment: urls.cigna.prod,

  initialize: function() {
    this.bindEvents();
  },

  log: function(msg) {
    if (this.debug && msg && window.console) {
      console.log(msg);
    }
  },

  preventBackButton: function(e) {
    e.preventDefault();
  },

  disableBackButton: function() {
    document.addEventListener('backbutton', this.preventBackButton, false);
  },

  enableBackButton: function() {
    document.removeEventListener('backbutton', this.preventBackButton);
  },

  startApp: function() {
    var content = document.getElementById('content');
    var appcacheLoaded = localStorage.getItem('appcache-loaded');

    if (navigator.connection.type === Connection.NONE && !appcacheLoaded) {
      // alert('No internet connection. Please check that your internet connection is active and try again.');
      content.src = 'offline.html';
      this.disableBackButton();
    } else {
      content.src = this.environment;
      this.enableBackButton();
    }
  },

  bindEvents: function() {
    document.addEventListener('deviceready', this.onDeviceReady.bind(this), false);
    document.addEventListener('online', this.startApp.bind(this), false);
    document.addEventListener('offline', this.startApp.bind(this), false);

    // Prevent overflow scroll on ios
    document.body.addEventListener('touchmove',function(e) {
      e.preventDefault();
    });

    window.addEventListener('message', this.receiveMessage.bind(this), false);
  },

  onDeviceReady: function() {
    this.log('Cordova loaded');

    this.deviceReady = true;
    this.cordova = ('undefined' != typeof(cordova) ? cordova : false) ||
                    ('undefined' != typeof(PhoneGap) ? PhoneGap : false) ||
                    ('undefined' != typeof(phonegap) ? phonegap : false);

    this.startApp();

    // Let's try 10 times, last resort fix!
    for (var i = 0; i < 10; i++) {
      setTimeout(this.setWebviewBackgroundColor.bind(this), 1000 * i);
    }
  },

  saveBase64Image: function(data, source) {
    var filename = data && data.filename;
    var base64 = data && data.base64;
    var self = this;

    this.log('Saving base64 image.');

    if (this.deviceReady && window.ImageToAlbumPlugin && filename && base64) {
      var directory = 'Cigna';
      var filename = filename + '.png';

      this.log('Saving received base64 image.');

      window.ImageToAlbumPlugin.saveImageDataToLibrary(
        function(msg) {
          self.log('Base64 image saved: ' + msg + '.');
          source.postMessage({ method: 'saveBase64Image', error: false }, '*');
        }, function(err) {
          self.log('Base64 image not saved: ' + err + '.');
          source.postMessage({ method: 'saveBase64Image', error: err }, '*');
        },
        base64, directory, filename
      );
    } else {
      // throw new Error('File not specified.');
    }
  },

  saveContactInformation: function(data, source) {
    this.log('Saving contact information.');

    if (data) {
      this.log('Saving received contact information.');
      window.localStorage && localStorage.setItem('contact-information', data);
    }
  },

  loadContactInformation: function(data, source) {
    var information = window.localStorage && localStorage.getItem('contact-information');

    console.log('Loading contact information.');

    if (information) {
      this.log('Sending loaded contact information.');
      source.postMessage({ method: 'loadContactInformation', information: information }, '*');
    } else {
      this.log('No contact information found.');
    }
  },

  hideSplashScreen: function(data, source) {
    this.log('Hiding splashscreen.');
    navigator.splashScreen && navigator.splashscreen.hide();
  },

  setWebviewBackgroundColor: function() {
    this.log('Setting webview background color.');

    try {
      window.plugins && plugins.webviewcolor && plugins.webviewcolor.change('#efefef');
    } catch(err) {
      console.log(err);
    }
  },

  setLoggedInStatusBar: function(data, source) {
    this.log('Setting logged in statusbar.');
    window.StatusBar && window.StatusBar.backgroundColorByHexString('#004986');
  },

  appcacheLoaded: function() {
    this.log('Setting appcache-loaded flag in localStorage');
    var appcacheLoaded = localStorage.getItem('appcache-loaded');

    if (!appcacheLoaded) {
      localStorage.setItem('appcache-loaded', true);
    }
  },

  hasActiveInternetConnection: function(data, source) {
    this.log('Checking active internet connection');

    var active = navigator.connection.type !== Connection.NONE;
    var message = "No internet connection. Please check that your internet connection is active and try again.";

    if (localStorage.getItem('contact-information')) {
      message = 'No internet connection. Please check that your internet connection is active and try again, or contact us.';
    }

    source.postMessage({ method: 'hasActiveInternetConnection', active: active, message: message }, '*');
  },

  openUrlInBrowser: function(url, source) {
    this.log('Opening url in system browser: ' + url);

    var platform = window.device ? window.device.platform + '' : 'browser';

    this.log('platform: ' + platform);;

    if (platform && url) {
      if (platform.toUpperCase() === 'IOS') {
        window.cordova.InAppBrowser.open(url, '_system');
      } else if (platform.toUpperCase() === 'ANDROID') {
        window.open(url, '_system');
      }
    }
  },

  getFileUploadUrl: function (base, filename, data) {
    var _self = this;

    var query = '';

    Object.keys(data)
      .forEach(function (key, index) {
        if (index > 0) {
          query += '&';
        }

        query += encodeURIComponent(key) + '=' + encodeURIComponent(data[key]);
      });

    if (Object.keys(data).length > 0) {
      query += '&';
    }

    query += encodeURIComponent('fileName') + '=' + encodeURIComponent(filename);

    if (base.indexOf('#') > -1) {
      base = base.split('#')[0];
    }

    if (base.indexOf('?') > -1) {
      query = '&' + query;
    } else {
      query = '?' + query;
    }

    return base + query;
  },

  addPhoto: function (data, source) {
    if (window.cignaOC) {
      var _self = this;
      var cignaOC = window.cignaOC;

      var fail = function (type, extraData) {
        source.postMessage({ method: 'addPhoto', type: (type || 'upload_error'), target: data.target, extraData: extraData }, '*');
      };

      var maxFileSize = 1024 * 1024 * 10;

      var imageDirectory = this.photoDirectory();

      cignaOC.captureImage(function (response) {
        if (response.success) {
          var path = response.path;
          var mimeType = response.mimeType;
          var fileName = response.fileName;

          window.resolveLocalFileSystemURL(path.indexOf('://') > -1 ? path : 'file://' + path, function (fileEntry) {
            fileEntry.file(function(file) {
              if (file.size > maxFileSize) {
                // File is too large..
                fail('upload_error_filesize_max', {
                  filesize: maxFileSize,
                  filename: fileName
                });
              } else {
                // All good, upload it
                var fileUploadUrl = _self.getFileUploadUrl(data.action, fileName, data.data);

                var photo = {
                  id: app.totalPhotos,
                  url: path
                }

                app.claimPhotos.push(photo);
                app.totalPhotos += 1;

                // Upload photo
                source.postMessage({ method: 'addPhoto', type: 'upload_start', target: data.target, result: fileName }, '*');

                var options = new FileUploadOptions();

                options.fileKey = 'fileStream';
                options.fileName = fileName;
                options.mimeType = mimeType;
                options.httpMethod = data.method.toUpperCase();
                options.headers = {
                  'Content-Type': 'application/octet-stream'
                };

                options.params = data.data;

                var trustAllHosts = true; // Needed for android self-signed certificates

                var ft = new FileTransfer();

                ft.upload(path,
                  fileUploadUrl,
                  function (response) {
                    // Success
                    if (response && response.responseCode === 200) {
                      source.postMessage({ method: 'addPhoto', type: 'upload_success', target: data.target, result: response.response }, '*');
                    } else {
                      fail();
                    }
                  }, fail, options, trustAllHosts);
              }
            }, fail);
          }, fail);
        } else {
          fail();
        }
      }, fail, imageDirectory);
    }
  },

  removePhoto: function (photo, source) {
    var index = app.claimPhotos.findIndex(function (element, index, array) {
      if (element.id == photo.id) {
        return true;
      }
    });

    if (index !== -1) {
        app.claimPhotos.splice(index, 1);

        window.resolveLocalFileSystemURL(photo.url, function (entry) {
          entry.remove(function () {

          }, function (e) {

          });

        }, function (e) {

        });
    }

    source.postMessage({ method: 'removePhoto', error: null, 'photo': photo }, '*');
  },

  fetchPhotoData: function (photo, source){
    window.resolveLocalFileSystemURL(photo.url, function (entry) {
      entry.file(function (file) {
        var reader = new FileReader();

        reader.onloadend = function () {
          //TODO: change type to image/png once we actually upload a png
          var blob = new Blob([new Uint8Array(this.result)], { type: 'text/plain;charset=UTF-8' });

          source.postMessage({ method: 'photoDataFetched', error: null, photo: photo, data: blob }, '*');
        };

        reader.readAsArrayBuffer(file);
      });
    });
  },

  downloadAndOpenFile: function (data, source) {
    var fail = function (error) {
      source.postMessage({ method: 'hideLoader' }, '*');
      alert('Could not download file.');
      console.warn(error);
    };

    var platform = window.device ? window.device.platform + '' : 'browser';

    var uri = encodeURI(data.url);
    var token = data.token;
    var username = data.username;

    var directory = 'cigna';
    var name = data.filename;

    var opts = {
      headers: {
        'X-Token': token,
        'X-Username': username
      }
    };

    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fileSystem) {
      fileSystem.root.getDirectory('cigna', { create: true, exclusive: false }, function (dirEntry) {
        dirEntry.getFile(name, { create: true, exclusive: false }, function (fileEntry) {
          var fileTransfer = new FileTransfer();

          fileTransfer.download(uri, fileEntry.nativeURL, function (entry) {
            source.postMessage({ method: 'hideLoader' }, '*');

            if (platform.toUpperCase() === 'IOS') {
              window.cordova.InAppBrowser.open(fileEntry.nativeURL, '_blank', 'location=no,enableViewportScale=yes');
            } else {
              window.open(fileEntry.nativeURL.replace('/storage/emulated/0', '/sdcard'), '_system', 'location=no,enableViewportScale=yes');
            }
          }, fail, true, opts);
        }, fail);
      }, fail);
    }, fail);
  },

  appVersion: function (data, source) {
    source.postMessage({ method: 'appVersion', result: window.AppVersion }, '*');
  },

  receiveMessage: function(evt) {
    var app = window.app;

    this.log('Received event: ' + evt.data.method + '.');

    if (!app.cordova) {
      //throw new Error('Cordova not yet loaded.');
      //return;
    }

    if (app && app[evt.data && evt.data.method]) {
      app[evt.data.method](evt.data.data, evt.source);
    }
  },

  photoDirectory: function(){
    return cordova.file.cacheDirectory;
  }
};

app.initialize();
